

# The following script contains code to recreate analyses of size and abundance data collected in 2015-2016 by researchers from the Marine Global Earth Observatory (MarineGEO). All code is original. Basic analysis steps are as follows:

#               > calculate fish densities (fish/m^2)
#               > calculate log-transformed and centered size-spectra for every survey
#               > fit models of size-spectra intercept and slope according to AICc and residual diagnostics tests
#               > define function to create models and perform Dunnett's test of structured habitat effects vs. soft sediment                       effects by location
#               > define function to calculate and clean estimated marginal means from slope and intercept models.
#               > run both functions on size-spectra dataframe to obtain Dunnett's test results and model-predicted responses.                       calculate 95% confidence intervals.

 

# Author: Jack C. Olson, Smithsonian Institution, Tenenbaum Marine Observatory Network, Marine Global Earth Observatory
# Date: 04/18/2022
# email: Olson.jackc@gmail.com



## load libraries

library(tidyverse)
library(janitor)
library(emmeans)
library(MuMIn)
library(DHARMa)
library(tidyverse)
library(dplyr)
library(lme4)
library(ggplot2)
library(emmeans)
library(broom)
library(lubridate)

options(scipen = 999)

## read in raw fish survey data with lengths and abundances estimated using DIDSON host software (Sound Metrics Corporation)

sonar_fish <- read.csv("SI_DIDSON_fish_data_2015_2016.csv")

### Columns in 'sonar_fish' are as follows:

#   'id' = vector of survey identifiers with location, sampling site, habitat type, and transect number
#   'date' = date when DIDSON sonar surveys were conducted
#   'location' = geographic sampling location. 'bocas' = Bocas del Toro, Panama; 'sms' = Fort Pierce, Florida; 'rb' = Richardson Bay CA; 'rr' = Rhode River, Maryland.
#   'site'= sampling site name
#   'habitat'= habitat type sampled. Note: "soft" denotes soft sediment and "reef" denotes "coral reef"
#   'transect_no' = replicate transect number completed per site and location
#   'size_class' = size class midoints in mm. Size classes were: 110-125mm, 125-150mm, 150-175mm, 175-200mm, 200-225mm, 225-250mm, 250-275mm, 275-300mm, 300-325mm, 325-350mm, 350-375mm, 375-400mm
#   'area_m2' = two dimensional area (m^2) of the benthos surveyed by sonar in each transect. Calculated from the sonar beam with at intersection with the bottom and length of transect. 
#   'number' = number of fish counted in each survey, by size class



## calculate fish density per m^2 for each sonar survey

sonar_fish <- sonar_fish %>%
  group_by(id,location, site, habitat, transect_no, size_class,area_m2,number) %>% 
  summarise(dens_m2 = number/area_m2) %>% 
  ungroup()


## ln(x +1) transform the density data, and ln() transform the size class midpoints (per Daan et al. 2005)

log_spectra <- sonar_fish %>% 
  group_by(location, site, habitat, transect_no, size_class) %>%
  mutate(ln_dens = log(dens_m2+1),log_size = log(size_class)) %>% 
  ungroup()


## center the size class midpoints

log_spectra$scale_sizes <- scale(log_spectra$log_size, scale = F)



## pair down data to only neccessary columns

log_spectra <- log_spectra %>% 
  select(location,site,habitat,transect_no,scale_sizes,log_size,ln_dens)


# create nested data frame

lm_nest <- log_spectra %>% 
  group_by(location,site,habitat,transect_no) %>% 
  nest()

## define lm function
library(broom)

## define function to create linear regression of ln(density + 1) by ln-transformed and centered size class midpoints

spectra_model <- function(df){
  lm(ln_dens ~ scale_sizes,data = df)
}


## turn of scientific notation

options(scipen = 999)

## apply linear model function over nested data frame and add within df with mutate

lm_nest <- lm_nest %>% 
  mutate(model = map(data,spectra_model)) %>% 
  mutate(model = map(model,tidy)) %>% 
  unnest(model)

## remove nested data column and other unneccessary columns
lm_nest <- lm_nest %>% 
  dplyr::select(-c(data,p.value,std.error,statistic))

## create vectors for spectra intercept '(Intercept)' and slope ('scale_sizes')
lm_nest <- spread(lm_nest, key = "term", value = "estimate")


# rename columns
lm_nest <- lm_nest %>% 
  rename(Intercept = `(Intercept)`,
         size_slope = scale_sizes)


#*********************************************************************************************************************
#*********************************************************************************************************************

## slope model fitting ##

## define cube root function for transformation of slopes 

CubeRoot<-function(x){
  sign(x)*abs(x)^(1/3)
}
# 

## create cube root slope column. Prior anaylses identified cube transformation as the most appropriate to satisfy the assumption of normality of errors and homoscedascity. 

lm_nest <- lm_nest %>%
  mutate(cube_rt_slope = CubeRoot(size_slope))


# change "seagrass" to "sav" within habitat column

lm_nest <- lm_nest %>%
  mutate(habitat = case_when(
    habitat == "seagrass" ~"sav",
    TRUE ~ habitat
  ))


mod1 <- lmer(cube_rt_slope ~ location/habitat + (1|site) + (1|transect_no), data = lm_nest)
AICc(mod1)
#38.7581

simres1 <- simulateResiduals(mod1)
plot(simres1)


#******************
mod1.1 <- lmer(cube_rt_slope ~ location/habitat + (1|site/transect_no), data = lm_nest)
AICc(mod1.1)

# singular fit -- over parameterized 


#***********************
mod1.2 <- lmer(cube_rt_slope ~ location/habitat + (1|site), data = lm_nest)
AICc(mod1.2)
#35.87216

simres1.2 <- simulateResiduals(mod1.2)
plot(simres1.2)
testDispersion(simres1.2)
### looks good 


mod1.3 <- lm(cube_rt_slope ~ location/habitat, data = lm_nest)
AICc(mod1.3)
#-21.72516 <<<< best

simres1.3 <- simulateResiduals(mod1.3)

plot(simres1.3)

# testDispersion(mod1.3)
# DHARMa nonparametric dispersion test via sd of residuals fitted vs. simulated
# 
# data:  simulationOutput
# dispersion = 0.86342, p-value = 0.304
# alternative hypothesis: two.sided

testZeroInflation(simres1.3)

# DHARMa zero-inflation test via comparison to expected zeros with simulation under H0 = fitted model
# 
# data:  simulationOutput
# ratioObsSim = NaN, p-value = 1
# alternative hypothesis: two.sided



##******************************************************************************************************************************
#********************************************************************************************************************************


## intercept model fitting 

#*****************************


mod1_int <- lmer(log(Intercept) ~ location/habitat + (1|site) + (1|transect_no), data = lm_nest)
## singular fit


#******************
mod1.1_int <- lmer(log(Intercept) ~ location/habitat + (1|site/transect_no), data = lm_nest)

# singular fit


#***********************
mod1.2_int <- lmer(log(Intercept) ~ location/habitat + (1|site), data = lm_nest)
# singular fit


mod1.3_int <- lm(log(Intercept) ~ location/habitat, data = lm_nest)
AICc(mod1.3_int)
#374.7611

simres1.3_int <- simulateResiduals(mod1.3_int)

plot(simres1.3_int)

testDispersion(mod1.3_int)
#data:  simulationOutput
#dispersion = 0.86342, p-value = 0.304
#alternative hypothesis: two.sided

testZeroInflation(simres1.3_int)

# DHARMa zero-inflation test via comparison to expected zeros with simulation under H0 = fitted model
# 
# data:  simulationOutput
# ratioObsSim = NaN, p-value = 1
# alternative hypothesis: two.sided




#***************************************************************************************************************************
#***************************************************************************************************************************
# Define function to calculate estimated marginal means by location and habitat from (previously fit) slope and intercept models, then run Dunnett's test of multiple treatments (structured habitats) against a single control (soft sediment). Clean the resulting model object using broom::tidy() which creates a model output table. 




nested_mod_fun <- function(df){
  # define and run models of slope and intercept
  slope_mod <- lm(CubeRoot(size_slope) ~ location/habitat, data = df)
  int_mod <- lm(log(Intercept) ~ location/habitat, data = df)
  
  # create emmGrid object
  slope_grid <- emmeans::emmeans(slope_mod, ~location/habitat)
  
  #run Dunnett's post hoc test by location on emmGrid slope object, with control ('ref') set as "soft" (soft sediment)
  
  slope_dun <- emmeans::contrast(slope_grid, method = "dunnett", ref = "soft", by = "location")
  
  ## same routine for intercept model
  int_grid <- emmeans::emmeans(int_mod,~location/habitat)
  int_dun <- emmeans::contrast(int_grid, method = "dunnett", ref = "soft", by = "location")
  
  ## clean output of Dunnett's test for each response variable
  int_dun <- broom::tidy(int_dun)
  slope_dun <- broom::tidy(slope_dun)
  
  # add response variable column to both data frames
  int_dun$response <- "intercept"
  slope_dun$response <- "slope"
  
  # bind data frames
  dunnetts_both <- bind_rows(slope_dun,int_dun)
  
  # round numeric columns to 4 digits
  dunnetts_both <- dunnetts_both %>%
    mutate_if(is.numeric, ~round(., 4))
  
  return(dunnetts_both)}


## Run function on nested data frame to produce results of Dunnett's test for both response variables across all locations and structured habitats. 

dunnetts_results_sonar_fish <- nested_mod_fun(lm_nest)



# Define function to calculated estimated marginal mean spectra slope and height by location and habitat for plotting. Use tidy() to create model table.  


nested_emmeans <- function(x){
  
  slope_mod <- lm(CubeRoot(size_slope) ~ location/habitat, data = x)
  int_mod <- lm(log(Intercept) ~ location/habitat, data = x)
  
  slope_grid <- emmeans(slope_mod,~location/habitat)
  int_grid <- emmeans(int_mod,~location/habitat)
  
  slope_grid <- tidy(slope_grid)
  int_grid <- tidy(int_grid)
  
  int_grid$response <- "intercept"
  slope_grid$response <- "slope"
  
  
  em_both <- bind_rows(slope_grid,int_grid)
  em_both <- em_both %>%
    mutate_if(is.numeric, ~round(., 4))
  
  
  return(em_both)}



# run nested emmeans function 
emmeans_sonar_fish <- nested_emmeans(lm_nest)


# calculate 95% confidence intervals of estimated marginal means

emmeans_sonar_fish$upr.cl <- emmeans_sonar_fish$estimate + (1.96 * emmeans_sonar_fish$std.error)
emmeans_sonar_fish$lwr.cl <- emmeans_sonar_fish$estimate - (1.96 * emmeans_sonar_fish$std.error)





